<?php
/**
 * 
 */
class WhileFotoOpen
{

	const VERSION = '0.1.0';

  private $host = "https://p.whilefoto.com/api";
	private $key;
	private $secret;
	private $token;
	private $shopid;

	public $debug =  false;
	public $cacheFile;

	public $errCode = 0;
	public $errMsg = "";

	public function __construct($options)
	{
		$this->key = $options['key'];
		$this->secret = $options['secret'];

		if(isset($options['cache_file'])) $this->cacheFile = $options['cache_file'];
		else $this->cacheFile = dirname(__FILE__)."/whilefoto-token.txt";

		/*
			生成 token的接口，有调用频繁的限制，每天不超过100次，因此需要本地做缓存
		 	从缓存中获取 token
		*/
		$cache = null;
		if(file_exists($this->cacheFile))
			$cache = json_decode(file_get_contents($this->cacheFile),true);
		// 没有token 或已过期 重新获取
		if($cache && $cache['expire_at'] > time()){
			$this->token = $cache['token'];
		}else{
			$res = $this->getToken();
			$cache = [
				'token'=>$res['token'],
				'expire_at'=>$res['expire_at'],
			];
			file_put_contents($this->cacheFile, json_encode($cache));
			$this->token = $res['token'];
		}
	}


	/*
	 * 获取当前商家信息
	 */
	public function getBrand(){
    $result = $this->_get("/v2/open/authed/brand");
    if($result) return $result['object'];
    return false;
	}

	/*
	 * 获取门店详情
	 */
	public function getShop($shop_id){
    $result = $this->_get("/v2/open/authed/shops/".$shop_id);
    if($result) return $result['object'];
    return false;
	}


	/*
	 * 获取商家的所有门店
	 */
	public function getShops(){
    $result = $this->_get("/v2/open/authed/shops");
    if($result) return $result['objects'];
    return false;
	}

	/*
	 * 获取门店的样片相册列表
	 */
	public function getShopAlbums($shop_id){
		$this->shopid = $shop_id;
    $result = $this->_get("/v2/open/authed/shops/shop/albums",['shop_id'=>$shop_id]);
    if($result) return $result['objects'];
    return false;
	}

	/*
	 * 获取门店的某个样片相册
	 */
	public function getShopAlbum($shop_id,$album_id){
		$this->shopid = $shop_id;
		$data = [
			'shop_id'=>$shop_id,
			'album_id'=>$album_id,
		];
    $result = $this->_get("/v2/open/authed/shops/shop/album",$data);
    if($result) return $result['objects'];
    return false;
	}


	/*
	 * 获取门店的所有套系
	 */
	public function getPackages($shop_id){
		$this->shopid = $shop_id;
    $result = $this->_get("/v2/open/authed/packages",['shop_id'=>$shop_id]);
    if($result) return $result['objects'];
    return false;
	}


	/*
	 * 获取门店的套系详情
	 */
	public function getPackage($shop_id, $package_id){
		$this->shopid = $shop_id;
    $result = $this->_get("/v2/open/authed/packages/$package_id",['shop_id'=>$shop_id]);
    if($result) return $result['object'];
    return false;
	}

	/*
	 * 获取商家拍摄类型
	 */
	public function getPhotoTypes(){
    $result = $this->_get("/v2/open/authed/photo_types");
    if($result) return $result['objects'];
    return false;
	}

	/*
	 * 获取订单信息
	 */
	public function getOrder($shop_id,$order_id){
		$this->shopid = $shop_id;
    $result = $this->_get("/v2/open/authed/orders/$order_id",['shop_id'=>$shop_id]);
    if($result) return $result['object'];
    return false;
	}


	/*
	 * 创建订单
	 */
	public function createOrder($shop_id,$data){
		$this->shopid = $shop_id;
    $result = $this->_post("/v2/open/authed/orders",$data);
    if($result) return $result['object'];
    return false;
	}




	/*
	 * 获取 api token
	 */
	public function getToken(){
		$data = [
    	"key"=>$this->key,
    	"timestamp"=>time(),
    ];
    $data["verify"] = md5($this->key.$this->secret.$data['timestamp']);
    $result = $this->_get("/v2/open/auth/token",$data);
    if($result) return $result;
    return false;
	}


	private function _get($api,$params=[]){
    $qs = http_build_query($params);
    $headers = [];
    if(isset($this->token))$headers[] = "Authorization: ".$this->token;
    if(isset($this->shopid))$headers[] = "Shop-Id: ".$this->shopid;
    $result = $this->http_get($this->host.$api."?".$qs, $headers);
		if ($result)
		{
			$json = json_decode($result,true);
			if (!$json || isset($json['error'])) {
				$this->errCode = $json['error'];
				$this->errMsg = $json['error'];
				return false;
			}
			return $json;
		}
		return false;
	}


	private function _post($api,$params=[]){
    $headers = [];
    if(isset($this->token))$headers[] = "Authorization: ".$this->token;
    if(isset($this->shopid))$headers[] = "Shop-Id: ".$this->shopid;
    $qs = http_build_query($params);
    $result = $this->http_post($this->host.$api,$qs,false, $headers);
		if ($result)
		{
			$json = json_decode($result,true);
			if (!$json || isset($json['error'])) {
				$this->errCode = $json['error'];
				$this->errMsg = $json['error'];
				return false;
			}
			return $json;
		}
		return false;
	}

  /**
   * 日志记录，可被重载。
   * @param mixed $log 输入日志
   * @return mixed
   */
  protected function log($log){
  		if ($this->debug && function_exists($this->logcallback)) {
  			if (is_array($log)) $log = print_r($log,true);
  			return call_user_func($this->logcallback,$log);
  		}
  }



	/**
	 * GET 请求
	 * @param string $url
	 */
	private function http_get($url,$headers=[]){
		$oCurl = curl_init();
		if(stripos($url,"https://")!==FALSE){
			curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, FALSE);
			curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, FALSE);
			curl_setopt($oCurl, CURLOPT_SSLVERSION, 1); //CURL_SSLVERSION_TLSv1

		}

		curl_setopt($oCurl, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($oCurl, CURLOPT_URL, $url);
		curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, 1 );
		$sContent = curl_exec($oCurl);
		$aStatus = curl_getinfo($oCurl);
		if ($sContent === false) {
			$this->errMsg = curl_error($oCurl);
		}
		curl_close($oCurl);

			return $sContent;
		if(intval($aStatus["http_code"])==200){
			return $sContent;
		}else{
			return false;
		}
	}

	/**
	 * POST 请求
	 * @param string $url
	 * @param array $param
	 * @param boolean $post_file 是否文件上传
	 * @return string content
	 */
	private function http_post($url,$param,$post_file=false,$headers=[]){
		$oCurl = curl_init();
		if(stripos($url,"https://")!==FALSE){
			curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, FALSE);
			curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($oCurl, CURLOPT_SSLVERSION, 1); //CURL_SSLVERSION_TLSv1
		}
		if (is_string($param) || $post_file) {
			$strPOST = $param;
		} else {
			$aPOST = array();
			foreach($param as $key=>$val){
				$aPOST[] = $key."=".urlencode($val);
			}
			$strPOST =  join("&", $aPOST);
		}
		curl_setopt($oCurl, CURLOPT_URL, $url);
		curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, 1 );
		curl_setopt($oCurl, CURLOPT_POST,true);
		curl_setopt($oCurl, CURLOPT_POSTFIELDS,$strPOST);
		curl_setopt($oCurl, CURLOPT_HTTPHEADER, $headers);
		$sContent = curl_exec($oCurl);
		$aStatus = curl_getinfo($oCurl);
		if ($sContent === false) {
			$this->errMsg = curl_error($oCurl);
		}
		curl_close($oCurl);
			return $sContent;
		if(intval($aStatus["http_code"])==200){
			return $sContent;
		}else{
			return false;
		}
	}

}

